@@ -1,9 +1,7 @@
 /*  
- *  (C) Copyright David Abrahams 2001. Permission to copy, use,
- *  modify, sell and distribute this software is granted provided this
- *  copyright notice appears in all copies. This software is provided
- *  "as is" without express or implied warranty, and with no claim as
- *  to its suitability for any purpose.
+ *  Copyright 2001-2004 David Abrahams.
+ *  Distributed under the Boost Software License, Version 1.0.
+ *  (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)
  */
 #ifndef FRAMES_DWA20011021_H
 # define FRAMES_DWA20011021_H
@@ -17,6 +15,9 @@ typedef struct frame FRAME;
 struct frame
 {
     FRAME* prev;
+    /** The nearest enclosing frame for which module->user_module is
+        true. */
+    FRAME* prev_user;
     LOL args[1];
     module_t* module;
     PARSE* procedure;
