@@ -121,6 +121,8 @@
 # include "make.h"
 # include "strings.h"
 # include "expand.h"
+# include "debug.h"
+# include "filesys.h"
 
 /* Macintosh is "special" */
 
@@ -144,7 +146,8 @@ struct globs globs = {
 # else
 	{ 0, 1 }, 		/* debug ... */
 # endif
-	0			/* output commands, not run them */
+	0,			/* output commands, not run them */
+    0 /* action timeout */
 } ;
 
 /* Symbols to be defined as true for use in Jambase */
@@ -196,6 +199,14 @@ static void run_unit_tests()
 }
 #endif
 
+#ifdef HAVE_PYTHON
+    extern PyObject*
+    bjam_call(PyObject *self, PyObject *args);
+ 
+    extern PyObject*
+    bjam_import_rule(PyObject* self, PyObject* args);
+#endif
+
 int  main( int argc, char **argv, char **arg_environ )
 {
     int		n;
@@ -206,6 +217,7 @@ int  main( int argc, char **argv, char **arg_environ )
     int		status;
     int arg_c = argc;
     char ** arg_v = argv;
+    const char *progname = argv[0];
 
 # ifdef OS_MAC
     InitGraf(&qd.thePort);
@@ -213,15 +225,16 @@ int  main( int argc, char **argv, char **arg_environ )
 
     argc--, argv++;
 
-	if( ( n = getoptions( argc, argv, "-:d:j:f:gs:t:ano:qv", optv ) ) < 0 )
+	if( getoptions( argc, argv, "-:l:d:j:f:gs:t:ano:qv", optv ) < 0 )
     {
-        printf( "\nusage: jam [ options ] targets...\n\n" );
+        printf( "\nusage: %s [ options ] targets...\n\n", progname );
 
         printf( "-a      Build all targets, even if they are current.\n" );
         printf( "-dx     Set the debug level to x (0-9).\n" );
         printf( "-fx     Read x instead of Jambase.\n" );
 	    /* printf( "-g      Build from newest sources first.\n" ); */
         printf( "-jx     Run up to x shell commands concurrently.\n" );
+        printf( "-lx     Limit actions to x number of seconds after which they are stopped.\n" );
         printf( "-n      Don't actually execute the updating actions.\n" );
         printf( "-ox     Write the updating actions to file x.\n" );
 		printf( "-q      Quit quickly as soon as a target fails.\n" );
@@ -233,8 +246,6 @@ int  main( int argc, char **argv, char **arg_environ )
         exit( EXITBAD );
     }
 
-    argc -= n, argv += n;
-
     /* Version info. */
 
     if( ( s = getoptval( optv, 'v', 0 ) ) )
@@ -244,7 +255,8 @@ int  main( int argc, char **argv, char **arg_environ )
 	   printf( "   Copyright 1993-2002 Christopher Seiwald and Perforce Software, Inc.  \n" );
         printf( "   Copyright 2001 David Turner.\n" );
         printf( "   Copyright 2001-2004 David Abrahams.\n" );
-        printf( "   Copyright 2002-2004 Rene Rivera.\n" );
+        printf( "   Copyright 2002-2005 Rene Rivera.\n" );
+        printf( "   Copyright 2003-2005 Vladimir Prus.\n" );
 
         return EXITOK;
     }
@@ -265,6 +277,9 @@ int  main( int argc, char **argv, char **arg_environ )
 	if( ( s = getoptval( optv, 'g', 0 ) ) )
 	    globs.newestfirst = 1;
 
+    if( ( s = getoptval( optv, 'l', 0 ) ) )
+        globs.timeout = atoi( s );
+
     /* Turn on/off debugging */
 
     for( n = 0; s = getoptval( optv, 'd', n ); n++ )
@@ -294,6 +309,28 @@ int  main( int argc, char **argv, char **arg_environ )
             globs.debug[i--] = 1;
     }
 
+    { PROFILE_ENTER(MAIN);
+
+    #ifdef HAVE_PYTHON
+    {
+        PROFILE_ENTER(MAIN_PYTHON);
+        Py_Initialize();
+    
+        {
+            static PyMethodDef BjamMethods[] = {
+                {"call", bjam_call, METH_VARARGS,
+                 "Call the specified bjam rule."},
+                {"import_rule", bjam_import_rule, METH_VARARGS,
+                 "Imports Python callable to bjam."},
+                {NULL, NULL, 0, NULL}
+            };
+    
+            Py_InitModule("bjam", BjamMethods);
+        }
+        PROFILE_EXIT(MAIN_PYTHON);
+    }
+    #endif
+    
 #ifndef NDEBUG
     run_unit_tests();
 #endif
@@ -319,16 +356,11 @@ int  main( int argc, char **argv, char **arg_environ )
     }
 
  
-    {
-   /* Pleace don't change the following line. The 'bump_version.py' script
-       expect a specific format of it. */
-    char  *major_version = "03", *minor_version = "01", *changenum = "10";
     var_set( "JAM_VERSION",
-             list_new( list_new( list_new( L0, newstr( major_version ) ), 
-                                 newstr( minor_version ) ), 
-                       newstr( changenum ) ),
+             list_new( list_new( list_new( L0, newstr( VERSION_MAJOR_SYM ) ), 
+                                 newstr( VERSION_MINOR_SYM ) ), 
+                       newstr( VERSION_PATCH_SYM ) ),
              VAR_SET );
-    }
 
     /* And JAMUNAME */
 # ifdef unix
@@ -352,15 +384,25 @@ int  main( int argc, char **argv, char **arg_environ )
     }
 # endif /* unix */
 
-	/*
-	 * Jam defined variables OS, OSPLAT
-	 */
-
-    var_defines( othersyms );
-
     /* load up environment variables */
 
-    var_defines( use_environ );
+    /* first into global module, with splitting, for backward compatibility */
+    var_defines( use_environ, 1 );
+    
+    /* then into .ENVIRON, without splitting */
+    enter_module( bindmodule(".ENVIRON") );
+    var_defines( use_environ, 0 );
+    exit_module( bindmodule(".ENVIRON") );
+
+	/*
+	 * Jam defined variables OS, OSPLAT
+     * We load them after environment, so that
+     * setting OS in environment does not 
+     * change Jam notion of the current platform.
+	 */
+
+    var_defines( othersyms, 1 );
+
 
     /* Load up variables set on command line. */
 
@@ -369,7 +411,7 @@ int  main( int argc, char **argv, char **arg_environ )
         char *symv[2];
         symv[0] = s;
         symv[1] = 0;
-        var_defines( symv );
+        var_defines( symv, 1 );
     }
 
     /* Set the ARGV to reflect the complete list of arguments of invocation. */
@@ -385,9 +427,18 @@ int  main( int argc, char **argv, char **arg_environ )
 
     /* Add the targets in the command line to update list */
 
-    for ( n = 0; n < argc; ++n )
+    for ( n = 1; n < arg_c; ++n )
     {
-        mark_target_for_updating(argv[n]);
+        if ( arg_v[n][0] == '-' )
+        {
+            char *f = "-:l:d:j:f:gs:t:ano:qv";
+            for( ; *f; f++ ) if( *f == arg_v[n][1] ) break;
+            if ( f[1] == ':' && arg_v[n][2] == '\0' ) { ++n; }
+        }
+        else
+        {
+            mark_target_for_updating(arg_v[n]);
+        }
     }
 
     /* Parse ruleset */
@@ -424,6 +475,8 @@ int  main( int argc, char **argv, char **arg_environ )
     /* Now make target */
 
     {
+        PROFILE_ENTER(MAIN_MAKE);
+        
         LIST* targets = targets_to_update();
         if ( !targets )
         {
@@ -434,6 +487,8 @@ int  main( int argc, char **argv, char **arg_environ )
             int targets_count = list_length(targets);
             const char **targets2 = (const char **)malloc(targets_count * sizeof(char *));
             int n = 0;
+            if ( DEBUG_PROFILE )
+                profile_memory( targets_count * sizeof(char *) );
             for ( ; targets; targets = list_next(targets) )
             {
                 targets2[n++] = targets->string;
@@ -441,15 +496,20 @@ int  main( int argc, char **argv, char **arg_environ )
             status |= make( targets_count, targets2, anyhow );       
             free(targets);
         }
+        
+        PROFILE_EXIT(MAIN_MAKE);
     }
 
 
+    PROFILE_EXIT(MAIN); }
+    
     if ( DEBUG_PROFILE )
         profile_dump();
 
     /* Widely scattered cleanup */
 
     var_done();
+    file_done();
     donerules();
     donestamps();
     donestr();
@@ -459,5 +519,10 @@ int  main( int argc, char **argv, char **arg_environ )
     if( globs.cmdout )
         fclose( globs.cmdout );
 
+#ifdef HAVE_PYTHON
+    Py_Finalize();
+#endif
+
+
     return status ? EXITBAD : EXITOK;
 }
