@@ -37,6 +37,14 @@
 
 #ifndef JAM_H_VP_2003_08_01
 #define JAM_H_VP_2003_08_01
+
+#ifdef HAVE_PYTHON
+#include <Python.h>
+#endif
+
+/* Assume popen support is available unless known otherwise. */
+#define HAVE_POPEN 1
+
 /*
  * VMS, OPENVMS
  */
@@ -111,6 +119,12 @@
 # define OS_AS400
 # endif
 
+/* Metrowerks Standard Library on Windows. */
+
+# ifdef __MSL__
+#undef HAVE_POPEN
+#endif
+
 # endif
 
 /*
@@ -231,10 +245,14 @@
 # define OSMINOR "OS=CYGWIN"
 # define OS_CYGWIN
 # endif
-# ifdef __FreeBSD__
+# if defined(__FreeBSD__) && !defined(__DragonFly__)
 # define OSMINOR "OS=FREEBSD"
 # define OS_FREEBSD
 # endif
+# ifdef __DragonFly__
+# define OSMINOR "OS=DRAGONFLYBSD"
+# define OS_DRAGONFLYBSD
+# endif
 # ifdef __DGUX__
 # define OSMINOR "OS=DGUX"
 # define OS_DGUX
@@ -295,11 +313,11 @@
 # define NO_VFORK
 # endif
 # ifdef __QNX__
+# define unix
 # ifdef __QNXNTO__
 # define OSMINOR "OS=QNXNTO"
 # define OS_QNXNTO
 # else
-# define unix
 # define OSMINOR "OS=QNX"
 # define OS_QNX
 # define NO_VFORK
@@ -321,6 +339,9 @@
 # define OS_MACOSX
 # endif
 # ifdef __osf__
+# ifndef unix
+# define unix
+# endif
 # define OSMINOR "OS=OSF"
 # define OS_OSF
 # endif
@@ -364,6 +385,10 @@
 # define OS_OPENBSD
 # define unix
 # endif
+# if defined (__FreeBSD_kernel__) && !defined(__FreeBSD__)
+# define OSMINOR "OS=KFREEBSD"
+# define OS_KFREEBSD
+# endif
 # ifndef OSMINOR
 # define OSMINOR "OS=UNKNOWN"
 # endif
@@ -394,6 +419,7 @@
 
 # if !defined(OS_BSDI) && \
      !defined(OS_FREEBSD) && \
+     !defined(OS_DRAGONFLYBSD) && \
      !defined(OS_NEXT) && \
      !defined(OS_MACHTEN) && \
      !defined(OS_MACOSX) && \
@@ -426,6 +452,7 @@
      defined( __i386__ ) || \
      defined( _M_IX86 )
 # if !defined( OS_FREEBSD ) && \
+     !defined( OS_DRAGONFLYBSD ) && \
      !defined( OS_OS2 ) && \
      !defined( OS_AS400 )
 # define OSPLAT "OSPLAT=X86"
@@ -497,6 +524,7 @@ struct globs {
 	int	newestfirst;		/* build newest sources first */
 	char	debug[DEBUG_MAX];
 	FILE	*cmdout;		/* print cmds, not run them */
+    long timeout; /* number of seconds to limit actions to, default 0 for no limit. */
 } ;
 
 extern struct globs globs;
